@@ -92,6 +92,7 @@ class Host(object):
         self.arch = None
 
         self.features = CapabilityFeatures()
+        self.topology = None
 
         if not node is None:
             self.parseXML(node)
@@ -99,6 +100,9 @@ class Host(object):
     def parseXML(self, node):
         child = node.children
         while child:
+            if child.name == "topology":
+                self.topology = Topology(child)
+
             if child.name != "cpu":
                 child = child.next
                 continue
@@ -160,6 +164,8 @@ class Guest(object):
 
 
     def bestDomainType(self, accelerated=None):
+        if len(self.domains) == 0:
+            raise CapabilitiesParserException(_("No domains available for this guest."))
         if accelerated is None:
             # Picking last in list so we favour KVM/KQEMU over QEMU
             return self.domains[-1]
@@ -201,10 +207,52 @@ class Domain(object):
         if len(machines) > 0:
             self.machines = machines
 
+class Topology(object):
+    def __init__(self, node = None):
+        self.cells = []
+
+        if not node is None:
+            self.parseXML(node)
+
+    def parseXML(self, node):
+        child = node.children
+        if child.name == "cells":
+            for cell in child.children:
+                if cell.name == "cell":
+                    self.cells.append(TopologyCell(cell))
+
+class TopologyCell(object):
+    def __init__(self, node = None):
+        self.id = None
+        self.cpus = []
+
+        if not node is None:
+            self.parseXML(node)
+
+    def parseXML(self, node):
+        self.id = int(node.prop("id"))
+        child = node.children
+        if child.name == "cpus":
+            for cpu in child.children:
+                if cpu.name == "cpu":
+                    self.cpus.append(TopologyCPU(cpu))
+
+class TopologyCPU(object):
+    def __init__(self, node = None):
+        self.id = None
+
+        if not node is None:
+            self.parseXML(node)
+
+    def parseXML(self, node):
+        self.id = int(node.prop("id"))
+
+
 class Capabilities(object):
     def __init__(self, node = None):
         self.host = None
         self.guests = []
+        self._topology = None
 
         if not node is None:
             self.parseXML(node)
@@ -213,11 +261,18 @@ class Capabilities(object):
         self._fixBrokenEmulator()
 
     def guestForOSType(self, type = None, arch = None):
-        for g in self.guests:
-            if (type is None or g.os_type == type) and (arch is None or g.arch == arch):
-                return g
-        return None
+        if self.host is None:
+            return None
 
+        if arch is None:
+            archs = [self.host.arch, None]
+        else:
+            archs = [arch]
+        for a in archs:
+            for g in self.guests:
+                if (type is None or g.os_type == type) and \
+                   (a is None or g.arch == a):
+                    return g
 
     # 32-bit HVM emulator path, on a 64-bit host is wrong due
     # to bug in libvirt capabilities. We fix by copying the
@@ -251,8 +306,15 @@ class Capabilities(object):
                 self.host = Host(child)
             elif child.name == "guest":
                 self.guests.append(Guest(child))
+            if child.name == "topology":
+                self._topology = Topology(child)
             child = child.next
 
+        # Libvirt < 0.4.1 placed topology info at the capabilities level
+        # rather than the host level. This is just for back compat
+        if self.host.topology is None:
+            self.host.topology = self._topology
+
 def parse(xml):
     class ErrorHandler:
         def __init__(self):
