@@ -29,6 +29,7 @@ import struct
 import tempfile
 import util
 import Guest
+from VirtualDisk import VirtualDisk
 from virtinst import _virtinst as _
 
 from ImageFetcher import MountedImageFetcher
@@ -39,11 +40,13 @@ from ImageFetcher import DirectImageFetcher
 from OSDistro import FedoraDistro
 from OSDistro import RHELDistro
 from OSDistro import CentOSDistro
+from OSDistro import SLDistro
 from OSDistro import SuseDistro
 from OSDistro import DebianDistro
 from OSDistro import UbuntuDistro
 from OSDistro import GentooDistro
 from OSDistro import MandrivaDistro
+from OSDistro import GenericDistro
 
 def _fetcherForURI(uri, scratchdir=None):
     if uri.startswith("http://"): 
@@ -58,25 +61,30 @@ def _fetcherForURI(uri, scratchdir=None):
         else:
             return MountedImageFetcher(uri, scratchdir)
 
-def _storeForDistro(fetcher, baseuri, type, progresscb, distro=None, scratchdir=None):
+def _storeForDistro(fetcher, baseuri, type, progresscb, arch, distro=None,
+                    scratchdir=None):
     stores = []
     logging.debug("Attempting to detect distro:")
     if distro == "fedora" or distro is None:
-        stores.append(FedoraDistro(baseuri, type, scratchdir))
+        stores.append(FedoraDistro(baseuri, type, scratchdir, arch))
     if distro == "rhel" or distro is None:
-        stores.append(RHELDistro(baseuri, type, scratchdir))
+        stores.append(RHELDistro(baseuri, type, scratchdir, arch))
     if distro == "centos" or distro is None:
-        stores.append(CentOSDistro(baseuri, type, scratchdir))
+        stores.append(CentOSDistro(baseuri, type, scratchdir, arch))
+    if distro == "sl" or distro is None:
+        stores.append(SLDistro(baseuri, type, scratchdir, arch))
     if distro == "suse" or distro is None:
-        stores.append(SuseDistro(baseuri, type, scratchdir))
+        stores.append(SuseDistro(baseuri, type, scratchdir, arch))
     if distro == "debian" or distro is None:
-        stores.append(DebianDistro(baseuri, type, scratchdir))
+        stores.append(DebianDistro(baseuri, type, scratchdir, arch))
     if distro == "ubuntu" or distro is None:
-        stores.append(UbuntuDistro(baseuri, type, scratchdir))
+        stores.append(UbuntuDistro(baseuri, type, scratchdir, arch))
     if distro == "gentoo" or distro is None:
-        stores.append(GentooDistro(baseuri, type, scratchdir))
+        stores.append(GentooDistro(baseuri, type, scratchdir, arch))
     if distro == "mandriva" or distro is None:
-        stores.append(MandrivaDistro(baseuri, type, scratchdir))
+        stores.append(MandrivaDistro(baseuri, type, scratchdir, arch))
+
+    stores.append(GenericDistro(baseuri, type, scratchdir, arch))
 
     for store in stores:
         if store.isValidStore(fetcher, progresscb):
@@ -86,7 +94,8 @@ def _storeForDistro(fetcher, baseuri, type, progresscb, distro=None, scratchdir=
 
 
 # Method to fetch a kernel & initrd pair for a particular distro / HV type
-def acquireKernel(baseuri, progresscb, scratchdir="/var/tmp", type=None, distro=None):
+def acquireKernel(baseuri, progresscb, scratchdir="/var/tmp", type=None,
+                  distro=None, arch=None):
     fetcher = _fetcherForURI(baseuri, scratchdir)
     
     try:
@@ -95,14 +104,16 @@ def acquireKernel(baseuri, progresscb, scratchdir="/var/tmp", type=None, distro=
         raise ValueError, _("Invalid install location: ") + str(e)
 
     try:
-        store = _storeForDistro(fetcher=fetcher, baseuri=baseuri, type=type, \
-                                progresscb=progresscb, distro=distro, scratchdir=scratchdir)
+        store = _storeForDistro(fetcher=fetcher, baseuri=baseuri, type=type,
+                                progresscb=progresscb, distro=distro,
+                                scratchdir=scratchdir, arch=arch)
         return store.acquireKernel(fetcher, progresscb)
     finally:
         fetcher.cleanupLocation()
 
 # Method to fetch a bootable ISO image for a particular distro / HV type
-def acquireBootDisk(baseuri, progresscb, scratchdir="/var/tmp", type=None, distro=None):
+def acquireBootDisk(baseuri, progresscb, scratchdir="/var/tmp", type=None,
+                    distro=None, arch=None):
     fetcher = _fetcherForURI(baseuri, scratchdir)
 
     try:
@@ -111,25 +122,34 @@ def acquireBootDisk(baseuri, progresscb, scratchdir="/var/tmp", type=None, distr
         raise ValueError, _("Invalid install location: ") + str(e)
 
     try:
-        store = _storeForDistro(fetcher=fetcher, baseuri=baseuri, type=type, \
-                                progresscb=progresscb, distro=distro, scratchdir=scratchdir)
+        store = _storeForDistro(fetcher=fetcher, baseuri=baseuri, type=type,
+                                progresscb=progresscb, distro=distro,
+                                scratchdir=scratchdir, arch=arch)
         return store.acquireBootDisk(fetcher, progresscb)
     finally:
         fetcher.cleanupLocation()
 
 class DistroInstaller(Guest.Installer):
-    def __init__(self, type = "xen", location = None, boot = None, extraargs = None, os_type = None):
-        Guest.Installer.__init__(self, type, location, boot, extraargs, os_type)
+    def __init__(self, type = "xen", location = None, boot = None,
+                 extraargs = None, os_type = None, conn = None):
+        Guest.Installer.__init__(self, type, location, boot, extraargs,
+                                 os_type, conn=conn)
 
     def get_location(self):
         return self._location
     def set_location(self, val):
         # 'location' is kind of overloaded: it can be a local file or device
-        # (for a boot.iso), local directory (for a tree), or an http, ftp, or 
+        # path (for a boot.iso), a local directory (for a tree), a
+        # tuple of the form (poolname, volname), or an http, ftp, or
         # nfs for an iso or a tree
-        if os.path.exists(os.path.abspath(val)):
+        if type(val) is tuple and len(val) == 2:
+            logging.debug("DistroInstaller location is a (poolname, volname)"
+                          " tuple")
+        elif os.path.exists(os.path.abspath(val)) \
+             and (not self.conn or not util.is_uri_remote(self.conn.getURI())):
             val = os.path.abspath(val)
-            logging.debug("DistroInstaller location is a local file/path: %s" % val)
+            logging.debug("DistroInstaller location is a local "
+                          "file/path: %s" % val)
         elif val.startswith("nfs://"):
             # Convert RFC compliant NFS      nfs://server/path/to/distro
             # to what mount/anaconda expect  nfs:server:/path/to/distro
@@ -141,33 +161,58 @@ class DistroInstaller(Guest.Installer):
             if index == -1:
                 raise ValueError(_("Invalid NFS format: No path specified."))
             if val[index - 1] != ":":
-                val = val[:index - 1] + ":" + val[index:] 
+                val = val[:index] + ":" + val[index:] 
 
-        elif not (val.startswith("http://") or val.startswith("ftp://") or
-                  val.startswith("nfs:")):
-            raise ValueError(_("Install media location must be an NFS, HTTP or FTP network install source, or an existing local file/device"))
+        elif (val.startswith("http://") or val.startswith("ftp://") or
+              val.startswith("nfs:")):
+            logging.debug("DistroInstaller location is a network source.")
+        elif self.conn and util.is_storage_capable(self.conn) and \
+             util.is_uri_remote(self.conn.getURI()):
+            # If conn is specified, pass the path to a VirtualDisk object
+            # and see what comes back
+            try:
+                v = VirtualDisk(path=val, device=VirtualDisk.DEVICE_CDROM,
+                                conn=self.conn)
+            except Exception, e:
+                raise ValueError(_("Checking installer location failed: %s" %\
+                                 str(e)))
+        else:
+            raise ValueError(_("Install media location must be an NFS, HTTP "
+                               "or FTP network install source, or an existing "
+                               "local file/device"))
 
         if os.geteuid() != 0 and val.startswith("nfs:"):
             raise ValueError(_("NFS installations are only supported as root"))
+
         self._location = val
     location = property(get_location, set_location)
 
     def _prepare_cdrom(self, guest, distro, meter):
-        if self.location.startswith("/") and os.path.exists(self.location):
+        cdrom = None
+        vol_tuple = None
+        if type(self.location) is tuple:
+            vol_tuple = self.location
+        elif self.location.startswith("/"):
             # Huzzah, a local file/device
             cdrom = self.location
         else:
             # Xen needs a boot.iso if its a http://, ftp://, or nfs: url
+            arch = os.uname()[4]
+            if hasattr(guest, "arch"):
+                arch = guest.arch
             cdrom = acquireBootDisk(self.location,
                                     meter,
                                     scratchdir = self.scratchdir,
-                                    distro = distro)
+                                    distro = distro,
+                                    arch = arch)
             self._tmpfiles.append(cdrom)
 
-        self._install_disk = Guest.VirtualDisk(cdrom,
-                                               device=Guest.VirtualDisk.DEVICE_CDROM,
-                                               readOnly=True,
-                                               transient=True)
+        self._install_disk = VirtualDisk(path=cdrom,
+                                         conn=guest.conn,
+                                         volName=vol_tuple,
+                                         device=VirtualDisk.DEVICE_CDROM,
+                                         readOnly=True,
+                                         transient=True)
 
     def _prepare_kernel_and_initrd(self, guest, distro, meter):
         if self.boot is not None:
@@ -179,11 +224,15 @@ class DistroInstaller(Guest.Installer):
         else:
             # Need to fetch the kernel & initrd from a remote site, or
             # out of a loopback mounted disk image/device
+            arch = os.uname()[4]
+            if hasattr(guest, "arch"):
+                arch = guest.arch
             (kernelfn, initrdfn, args) = acquireKernel(self.location,
                                                        meter,
                                                        scratchdir = self.scratchdir,
                                                        type = self.os_type,
-                                                       distro = distro)
+                                                       distro = distro,
+                                                       arch=arch)
             self.install["kernel"] = kernelfn
             self.install["initrd"] = initrdfn
             if not self.extraargs is None:
@@ -197,9 +246,9 @@ class DistroInstaller(Guest.Installer):
         # If they're installing off a local file/device, we map it
         # through to a virtual harddisk
         if self.location is not None and self.location.startswith("/") and not os.path.isdir(self.location):
-            self._install_disk = Guest.VirtualDisk(self.location,
-                                                   readOnly=True,
-                                                   transient=True)
+            self._install_disk = VirtualDisk(self.location,
+                                             readOnly=True,
+                                             transient=True)
 
     def prepare(self, guest, meter, distro = None):
         self.cleanup()
@@ -211,11 +260,15 @@ class DistroInstaller(Guest.Installer):
         }
 
         if self.cdrom:
-            self._prepare_cdrom(guest, distro, meter)
+            if self.location:
+                self._prepare_cdrom(guest, distro, meter)
+            else:
+                # Booting from a cdrom directly allocated to the guest
+                pass
         else:
             self._prepare_kernel_and_initrd(guest, distro, meter)
 
-    def _get_osblob(self, install, hvm, arch = None, loader = None):
+    def _get_osblob(self, install, hvm, arch = None, loader = None, conn = None):
         osblob = ""
         if install or hvm:
             osblob = "<os>\n"
@@ -245,11 +298,19 @@ class DistroInstaller(Guest.Installer):
 
             osblob += "  </os>"
         else:
-            osblob += "<bootloader>/usr/bin/pygrub</bootloader>"
+            osblob += "<bootloader>%s</bootloader>" % util.pygrub_path(conn)
 
         return osblob
 
     def post_install_check(self, guest):
+        if util.is_uri_remote(guest.conn.getURI()):
+            # Use block peek for this?
+            return True
+
+        if len(guest._install_disks) == 0 \
+           or guest._install_disks[0].device != VirtualDisk.DEVICE_DISK:
+            return True
+
         # Check for the 0xaa55 signature at the end of the MBR
         try:
             fd = os.open(guest._install_disks[0].path, os.O_RDONLY)
@@ -266,13 +327,15 @@ class DistroInstaller(Guest.Installer):
 
 
 class PXEInstaller(Guest.Installer):
-    def __init__(self, type = "xen", location = None, boot = None, extraargs = None, os_type = None):
-        Guest.Installer.__init__(self, type, location, boot, extraargs, os_type)
+    def __init__(self, type = "xen", location = None, boot = None,
+                 extraargs = None, os_type = None, conn = None):
+        Guest.Installer.__init__(self, type, location, boot, extraargs,
+                                 os_type, conn=conn)
 
     def prepare(self, guest, meter, distro = None):
         pass
 
-    def _get_osblob(self, install, hvm, arch = None, loader = None):
+    def _get_osblob(self, install, hvm, arch = None, loader = None, conn = None):
         osblob = ""
         if install or hvm:
             osblob = "<os>\n"
@@ -297,11 +360,19 @@ class PXEInstaller(Guest.Installer):
 
             osblob += "  </os>"
         else:
-            osblob += "<bootloader>/usr/bin/pygrub</bootloader>"
+            osblob += "<bootloader>%s</bootloader>" % util.pygrub_path(conn)
 
         return osblob
 
     def post_install_check(self, guest):
+        if util.is_uri_remote(guest.conn.getURI()):
+            # Use block peek for this?
+            return True
+
+        if len(guest._install_disks) == 0 \
+           or guest._install_disks[0].device != VirtualDisk.DEVICE_DISK:
+            return True
+
         # Check for the 0xaa55 signature at the end of the MBR
         try:
             fd = os.open(guest._install_disks[0].path, os.O_RDONLY)
