@@ -20,6 +20,7 @@
 # MA 02110-1301 USA.
 
 import os, os.path
+import statvfs
 import stat, sys, time
 import re
 import libxml2
@@ -27,222 +28,20 @@ import urlgrabber.progress as progress
 import util
 import libvirt
 import __builtin__
+import CapabilitiesParser
+from VirtualDisk import VirtualDisk
 from virtinst import _virtinst as _
 
 import logging
 
-class VirtualDisk:
-    DRIVER_FILE = "file"
-    DRIVER_PHY = "phy"
-    DRIVER_TAP = "tap"
+class VirtualNetworkInterface(object):
 
-    DRIVER_TAP_RAW = "aio"
-    DRIVER_TAP_QCOW = "qcow"
-    DRIVER_TAP_VMDK = "vmdk"
+    TYPE_BRIDGE  = "bridge"
+    TYPE_VIRTUAL = "network"
+    TYPE_USER    = "user"
 
-    DEVICE_DISK = "disk"
-    DEVICE_CDROM = "cdrom"
-    DEVICE_FLOPPY = "floppy"
-
-    TYPE_FILE = "file"
-    TYPE_BLOCK = "block"
-
-    def __init__(self, path = None, size = None, transient=False, type=None, device=DEVICE_DISK, driverName=None, driverType=None, readOnly=False, sparse=True):
-        """@path is the path to the disk image.
-           @size is the size of the disk image in gigabytes."""
-        self.size = size
-        self.sparse = sparse
-        self.transient = transient
-        self.path = path
-        self._type = type
-        self._readOnly = readOnly
-        self._device = device
-        self._driverName = driverName
-        self._driverType = driverType
-        self.target = None
-       
-        if self.path is not None:
-            # Check that the basics are valid
-            if __builtin__.type(self.path) is not __builtin__.type("string"):
-                raise ValueError, _("The %s path must be a string or None.") % self._device
-            self.path = os.path.abspath(self.path)
-            if os.path.isdir(self.path):
-                raise ValueError, _("The %s path must be a file or a device, not a directory") % self._device
-            if not os.path.exists(os.path.dirname(self.path)):
-                raise ValueError, _("The specified path's root directory must exist.")
-
-            if (self._device == self.DEVICE_FLOPPY or \
-                self._device == self.DEVICE_CDROM) and \
-               not os.path.exists(self.path):
-                raise ValueError, _("The %s path must exist.") % self._device
-
-            # If no disk type specified, attempt to determine from path
-            if self._type is None:
-                if not os.path.exists(self.path):
-                    logging.debug(\
-                        "Disk path not found: Assuming file disk type.");
-                    self._type = VirtualDisk.TYPE_FILE
-                else:
-                    if stat.S_ISBLK(os.stat(self.path)[stat.ST_MODE]):
-                        logging.debug(\
-                            "Path is block file: Assuming Block disk type.");
-                        self._type = VirtualDisk.TYPE_BLOCK
-                    else:
-                        self._type = VirtualDisk.TYPE_FILE
-        
-            if self._type == VirtualDisk.TYPE_FILE:
-                if self.size is None and not os.path.exists(self.path):
-                    raise ValueError, \
-                        _("A size must be provided for non-existent disks")
-                if os.path.exists(self.path) and \
-                   stat.S_ISBLK(os.stat(self.path)[stat.ST_MODE]):
-                    raise ValueError, _("The specified path is a block device, not a regular file.")
-                if self.size is not None and \
-                   (__builtin__.type(self.size) is not __builtin__.type(1) and __builtin__.type(self.size) is not __builtin__.type(1.0)):
-                    raise ValueError, _("Disk size must be an int or a float.")
-                if self.size < 0 and self.size is not None:
-                    raise ValueError, _("Disk size must not be less than 0.")
-            elif self._type == VirtualDisk.TYPE_BLOCK:
-                if not os.path.exists(self.path):
-                    raise ValueError, \
-                          _("The specified block device does not exist.")
-                if not stat.S_ISBLK(os.stat(self.path)[stat.ST_MODE]):
-                    raise ValueError, \
-                          _("The specified path is not a block device.")
-
-        else:
-            # Only floppy or cdrom can be created w/o media
-            if device != self.DEVICE_FLOPPY and \
-               device != self.DEVICE_CDROM:
-                raise ValueError, _("Disk type '%s' requires a path") % device
-
-    def get_type(self):
-        return self._type
-    type = property(get_type)
-
-    def get_transient(self):
-        return self._transient
-    transient = property(get_transient)
-
-    def get_device(self):
-        return self._device
-    device = property(get_device)
-
-    def get_driver_name(self):
-        return self._driverName
-    driver_name = property(get_driver_name)
-
-    def get_driver_type(self):
-        return self._driverType
-    driver_type = property(get_driver_type)
-
-    def get_read_only(self):
-        return self._readOnly
-    read_only = property(get_read_only)
-
-    def setup(self, progresscb):
-        if self._type == VirtualDisk.TYPE_FILE and self.path is not None \
-           and not os.path.exists(self.path):
-            size_bytes = long(self.size * 1024L * 1024L * 1024L)
-            progresscb.start(filename=self.path,size=long(size_bytes), \
-                             text=_("Creating storage file..."))
-            fd = None
-            try: 
-                try:
-                    fd = os.open(self.path, os.O_WRONLY | os.O_CREAT)
-                    if self.sparse:
-                        os.lseek(fd, size_bytes, 0)
-                        os.write(fd, '\x00')
-                        progresscb.update(self.size)
-                    else:
-                        buf = '\x00' * 1024 * 1024 # 1 meg of nulls
-                        for i in range(0, long(self.size * 1024L)):
-                            os.write(fd, buf)
-                            progresscb.update(long(i * 1024L * 1024L))
-                except OSError, detail:
-                    raise RuntimeError, "Error creating diskimage " + self.path + ": " + detail.strerror
-            finally:
-                if fd is not None:
-                    os.close(fd)
-                progresscb.end(size_bytes)
-        # FIXME: set selinux context?
-
-    def get_xml_config(self, disknode):
-        typeattr = 'file'
-        if self.type == VirtualDisk.TYPE_BLOCK:
-            typeattr = 'dev'
-
-        ret = "    <disk type='%(type)s' device='%(device)s'>\n" % { "type": self.type, "device": self.device }
-        if not(self.driver_name is None):
-            if self.driver_type is None:
-                ret += "      <driver name='%(name)s'/>\n" % { "name": self.driver_name }
-            else:
-                ret += "      <driver name='%(name)s' type='%(type)s'/>\n" % { "name": self.driver_name, "type": self.driver_type }
-        if self.path is not None:
-            path = util.xml_escape(self.path)
-            ret += "      <source %(typeattr)s='%(disk)s'/>\n" % { "typeattr": typeattr, "disk": path }
-        if self.target is not None:
-            disknode = self.target
-        ret += "      <target dev='%(disknode)s'/>\n" % { "disknode": disknode }
-        if self.read_only:
-            ret += "      <readonly/>\n"
-        ret += "    </disk>\n"
-        return ret
-
-    def is_conflict_disk(self, conn):
-        vms = []
-        # get working domain's name
-        ids = conn.listDomainsID();
-        for id in ids:
-            try:
-                vm = conn.lookupByID(id)
-                vms.append(vm)
-            except libvirt.libvirtError:
-                # guest probably in process of dieing
-                logging.warn("Failed to lookup domain id %d" % id)
-        # get defined domain
-        names = conn.listDefinedDomains()
-        for name in names:
-            try:
-                vm = conn.lookupByName(name)
-                vms.append(vm)
-            except libvirt.libvirtError:
-                # guest probably in process of dieing
-                logging.warn("Failed to lookup domain name %s" % name)
-
-        count = 0
-        for vm in vms:
-            doc = None
-            try:
-                doc = libxml2.parseDoc(vm.XMLDesc(0))
-            except:
-                continue
-            ctx = doc.xpathNewContext()
-            try:
-                try:
-                    count += ctx.xpathEval("count(/domain/devices/disk/source[@dev='%s'])" % self.path)
-                    count += ctx.xpathEval("count(/domain/devices/disk/source[@file='%s'])" % self.path)
-                except:
-                    continue
-            finally:
-                if ctx is not None:
-                    ctx.xpathFreeContext()
-                if doc is not None:
-                    doc.freeDoc()
-        if count > 0:
-            return True
-        else:
-            return False
-
-    def __repr__(self):
-        return "%s:%s" %(self.type, self.path)
-
-# Back compat class to avoid ABI break
-class XenDisk(VirtualDisk):
-    pass
-
-class VirtualNetworkInterface:
-    def __init__(self, macaddr = None, type="bridge", bridge = None, network=None):
+    def __init__(self, macaddr=None, type=TYPE_BRIDGE, bridge=None,
+                 network=None, model=None):
 
         if macaddr is not None and \
            __builtin__.type(macaddr) is not __builtin__.type("string"):
@@ -257,30 +56,27 @@ class VirtualNetworkInterface:
         self.type = type
         self.bridge = bridge
         self.network = network
-        if self.type == "network":
+        self.model = model
+        if self.type == self.TYPE_VIRTUAL:
             if network is None:
                 raise ValueError, _("A network name was not provided")
-            if bridge != None:
-                raise ValueError, _("Bridge name is not required for %s") % ("type=network",)
-        elif self.type == "bridge":
-            if network != None:
-                raise ValueError, _("Network name is not required for %s") % ("type=bridge",)
-        elif self.type == "user":
-            if network != None:
-                raise ValueError, _("Network name is not required for %s") % ("type=bridge",)
-            if bridge != None:
-                raise ValueError, _("Bridge name is not required for %s") % ("type=network",)
+        elif self.type == self.TYPE_BRIDGE:
+            pass
+        elif self.type == self.TYPE_USER:
+            pass
         else:
             raise ValueError, _("Unknown network type %s") % (type,)
 
     def is_conflict_net(self, conn):
         """is_conflict_net: determines if mac conflicts with others in system
 
-           returns a two element tuple: 
+           returns a two element tuple:
                first element is True if fatal collision occured
                second element is a string description of the collision.
            Non fatal collisions (mac addr collides with inactive guest) will
            return (False, "description of collision")"""
+        if self.macaddr is None:
+            return (False, None)
         # get Running Domains
         ids = conn.listDomainsID();
         vms = []
@@ -306,20 +102,18 @@ class VirtualNetworkInterface:
         hostdevs = util.get_host_network_devices()
 
         if self.countMACaddr(vms) > 0:
-            return (True, _("The MAC address you entered is already in use by another virtual machine!"))
+            return (True, _("The MAC address you entered is already in use by another active virtual machine."))
         for (dummy, dummy, dummy, dummy, host_macaddr) in hostdevs:
             if self.macaddr.upper() == host_macaddr.upper():
-                return (True, _("The MAC address you entered conflicts with the physical NIC."))
+                return (True, _("The MAC address you entered conflicts with a device on the physical host."))
         if self.countMACaddr(inactive_vm) > 0:
-            return (False, _("The MAC address you entered is already in use by another inactive virtual machine!"))
+            return (False, _("The MAC address you entered is already in use by another inactive virtual machine."))
         return (False, None)
 
     def setup(self, conn):
-
-        # check conflict MAC address
         if self.macaddr is None:
             while 1:
-                self.macaddr = util.randomMAC()
+                self.macaddr = util.randomMAC(type=conn.getType().lower())
                 if self.is_conflict_net(conn)[1] is not None:
                     continue
                 else:
@@ -327,9 +121,7 @@ class VirtualNetworkInterface:
         else:
             ret, msg = self.is_conflict_net(conn)
             if msg is not None:
-                # Error message found
                 if ret is False:
-                    # Not fatal
                     logging.warning(msg)
                 else:
                     raise RuntimeError(msg)
@@ -338,23 +130,21 @@ class VirtualNetworkInterface:
             self.bridge = util.default_bridge()
 
     def get_xml_config(self):
-        if self.type == "bridge":
-            return ("    <interface type='bridge'>\n" + \
-                    "      <source bridge='%(bridge)s'/>\n" + \
-                    "      <mac address='%(mac)s'/>\n" + \
-                    "    </interface>\n") % \
-                    { "bridge": self.bridge, "mac": self.macaddr }
-        elif self.type == "network":
-            return ("    <interface type='network'>\n" + \
-                    "      <source network='%(network)s'/>\n" + \
-                    "      <mac address='%(mac)s'/>\n" + \
-                    "    </interface>\n") % \
-                    { "network": self.network, "mac": self.macaddr }
-        elif self.type == "user":
-            return ("    <interface type='user'>\n" + \
-                    "      <mac address='%(mac)s'/>\n" + \
-                    "    </interface>\n") % \
-                    { "mac": self.macaddr }
+        src_xml = ""
+        model_xml = ""
+        if self.type == self.TYPE_BRIDGE:
+            src_xml =   "      <source bridge='%s'/>\n" % self.bridge
+        elif self.type == self.TYPE_VIRTUAL:
+            src_xml =   "      <source network='%s'/>\n" % self.network
+
+        if self.model:
+            model_xml = "      <model type='%s'/>\n" % self.model
+
+        return "    <interface type='%s'>\n" % self.type + \
+               src_xml + \
+               "      <mac address='%s'/>\n" % self.macaddr + \
+               model_xml + \
+               "    </interface>"
 
     def countMACaddr(self, vms):
         count = 0
@@ -366,13 +156,10 @@ class VirtualNetworkInterface:
                 continue
             ctx = doc.xpathNewContext()
             try:
-                try:
-                    count += ctx.xpathEval("count(/domain/devices/interface/mac[@address='%s'])"
-                                           % self.macaddr.upper())
-                    count += ctx.xpathEval("count(/domain/devices/interface/mac[@address='%s'])"
-                                           % self.macaddr.lower())
-                except:
-                    continue
+                for mac in ctx.xpathEval("/domain/devices/interface/mac"):
+                    macaddr = mac.xpathEval("attribute::address")[0].content
+                    if util.compareMAC(self.macaddr, macaddr) == 0:
+                        count += 1
             finally:
                 if ctx is not None:
                     ctx.xpathFreeContext()
@@ -380,64 +167,117 @@ class VirtualNetworkInterface:
                     doc.freeDoc()
         return count
 
+class VirtualAudio(object):
+
+    MODELS = [ "es1370", "sb16", "pcspk" ]
+
+    def __init__(self, model):
+        self.model = model
+
+    def get_model(self):
+        return self._model
+    def set_model(self, new_model):
+        if type(new_model) != str:
+            raise ValueError, _("'model' must be a string, "
+                                " was '%s'." % type(new_model))
+        if not self.MODELS.count(new_model):
+            raise ValueError, _("Unsupported sound model '%s'" % new_model)
+        self._model = new_model
+    model = property(get_model, set_model)
+
+    def get_xml_config(self):
+        return "    <sound model='%s'/>" % self.model
+
 # Back compat class to avoid ABI break
 class XenNetworkInterface(VirtualNetworkInterface):
     pass
 
-class VirtualGraphics:
-    def __init__(self, *args):
-        self.name = ""
+class VirtualGraphics(object):
+
+    TYPE_SDL = "sdl"
+    TYPE_VNC = "vnc"
+
+    def __init__(self, type=TYPE_VNC, port=-1, listen=None, passwd=None,
+                 keymap=None):
+
+        if type != self.TYPE_VNC and type != self.TYPE_SDL:
+            raise ValueError(_("Unknown graphics type"))
+        self._type   = type
+        self.set_port(port)
+        self.set_keymap(keymap)
+        self.set_listen(listen)
+        self.set_passwd(passwd)
+
+    def get_type(self):
+        return self._type
+    type = property(get_type)
+
+    def get_keymap(self):
+        return self._keymap
+    def set_keymap(self, val):
+        if not val:
+            val = util.default_keymap()
+        if not val or type(val) != type("string"):
+            raise ValueError, _("Keymap must be a string")
+        if len(val) > 16:
+            raise ValueError, _("Keymap must be less than 16 characters")
+        if re.match("^[a-zA-Z0-9_-]*$", val) == None:
+            raise ValueError, _("Keymap can only contain alphanumeric, '_', or '-' characters")
+        self._keymap = val
+    keymap = property(get_keymap, set_keymap)
+
+    def get_port(self):
+        return self._port
+    def set_port(self, val):
+        if val is None:
+            val = -1
+        elif type(val) is not int \
+             or (val != -1 and (val < 5900 or val > 65535)):
+            raise ValueError, _("VNC port must be a number between 5900 and 65535, or -1 for auto allocation")
+        self._port = val
+    port = property(get_port, set_port)
+
+    def get_listen(self):
+        return self._listen
+    def set_listen(self, val):
+        self._listen = val
+    listen = property(get_listen, set_listen)
+
+    def get_passwd(self):
+        return self._passwd
+    def set_passwd(self, val):
+        self._passwd = val
+    passwd = property(get_passwd, set_passwd)
 
     def get_xml_config(self):
-        return ""
-
-# Back compat class to avoid ABI break
-class XenGraphics(VirtualGraphics):
-    pass
-
-class VNCVirtualGraphics(XenGraphics):
-    def __init__(self, *args):
-        self.name = "vnc"
-        if len(args) >= 1 and not args[0] is None:
-            if args[0] < 5900 or args[0] > 65535:
-                raise ValueError, _("Invalid value for vnc port, port number must be in between 5900 and 65535")
-            self.port = args[0]
-        else:
-            self.port = -1
-        if len(args) >= 2 and args[1]:
-            self.keymap = args[1]
-        else:
-            self.keymap = None
-
-    def get_xml_config(self):
-        if self.keymap == None:
-            keymapstr = ""
-        else:
-            keymapstr = "keymap='"+self.keymap+"' "
-        return "    <graphics type='vnc' port='%(port)d' %(keymapstr)s/>" % {"port":self.port, "keymapstr":keymapstr}
-
-# Back compat class to avoid ABI break
-class XenVNCGraphics(VNCVirtualGraphics):
-    pass
-
-class SDLVirtualGraphics(XenGraphics):
-    def __init__(self, *args):
-        self.name = "sdl"
-
-    def get_xml_config(self):
-        return "    <graphics type='sdl'/>"
-
-# Back compat class to avoid ABI break
-class XenSDLGraphics(SDLVirtualGraphics):
-    pass
+        if self._type == self.TYPE_SDL:
+            return "    <graphics type='sdl'/>"
+        keymapxml = ""
+        listenxml = ""
+        passwdxml = ""
+        if self.keymap:
+            keymapxml = " keymap='%s'" % self._keymap
+        if self.listen:
+            listenxml = " listen='%s'" % self._listen
+        if self.passwd:
+            passwdxml = " passwd='%s'" % self._passwd
+        xml = "    <graphics type='vnc' " + \
+                   "port='%(port)d'" % { "port" : self._port } + \
+                   "%(keymapxml)s"   % { "keymapxml" : keymapxml } + \
+                   "%(listenxml)s"   % { "listenxml" : listenxml } + \
+                   "%(passwdxml)s"   % { "passwdxml" : passwdxml } + \
+                   "/>"
+        return xml
 
 class Installer(object):
-    def __init__(self, type = "xen", location = None, boot = None, extraargs = None, os_type = None):
+    def __init__(self, type = "xen", location = None, boot = None,
+                 extraargs = None, os_type = None, conn = None):
         self._location = None
         self._extraargs = None
         self._boot = None
         self._cdrom = False
         self._os_type = os_type
+        self._conn = conn
         self._install_disk = None   # VirtualDisk that contains install media
 
         if type is None:
@@ -463,6 +303,10 @@ class Installer(object):
         return self._install_disk
     install_disk = property(get_install_disk)
 
+    def get_conn(self):
+        return self._conn
+    conn = property(get_conn)
+
     def get_type(self):
         return self._type
     def set_type(self, val):
@@ -477,14 +321,15 @@ class Installer(object):
 
     def get_scratchdir(self):
         if self.type == "xen":
-            return "/var/lib/xen"
-        return "/var/tmp"
+            if os.path.exists("/var/lib/xen"):
+                return "/var/lib/xen"
+        return "/var/lib/libvirt/boot"
     scratchdir = property(get_scratchdir)
 
     def get_cdrom(self):
         return self._cdrom
     def set_cdrom(self, enable):
-        if __builtin__.type(enable) is not __builtin__.type(True):
+        if enable not in [True, False]:
             raise ValueError, _("Guest.cdrom must be a boolean type")
         self._cdrom = enable
     cdrom = property(get_cdrom, set_cdrom)
@@ -533,12 +378,12 @@ class Guest(object):
         self._maxmemory = None
         self._vcpus = None
         self._cpuset = None
-        self._graphics = { "enabled": False }
-        self._keymap = None
-        
+        self._graphics_dev = None
+
         # Public device lists unaltered by install process
         self.disks = []
         self.nics = []
+        self.sound_devs = []
 
         # Device lists to use/alter during install process
         self._install_disks = []
@@ -547,9 +392,13 @@ class Guest(object):
         self.domain = None
         self.conn = connection
         if self.conn == None:
+            logging.debug("No conn passed to Guest, opening URI '%s'" % \
+                          hypervisorURI)
             self.conn = libvirt.open(hypervisorURI)
         if self.conn == None:
-            raise RuntimeError, _("Unable to connect to hypervisor, aborting installation!")
+            raise RuntimeError, _("Unable to connect to hypervisor, aborting "
+                                  "installation!")
+        self._caps = CapabilitiesParser.parse(self.conn.getCapabilities())
 
         self.disknode = None # this needs to be set in the subclass
 
@@ -575,8 +424,9 @@ class Guest(object):
             raise ValueError, _("System name must be a string greater than 0 and no more than 50 characters")
         if re.match("^[0-9]+$", val):
             raise ValueError, _("System name must not be only numeric characters")
-        if re.match("^[a-zA-Z0-9._-]+$", val) == None:
-            raise ValueError, _("System name can only contain alphanumeric, '_', '.', or '-' characters")
+        if re.match("^[A-Za-z0-9_.:/+-]+$", val) == None:
+            raise ValueError, _("System name can only contain: alphanumeric "
+                                "'_', '.', ':', '+', or '-' characters")
         self._name = val
     name = property(get_name, set_name)
 
@@ -628,9 +478,11 @@ class Guest(object):
         return self._vcpus
     def set_vcpus(self, val):
         maxvcpus = util.get_max_vcpus(self.conn, self.type)
-        if val < 1 or val > maxvcpus:
+        if type(val) is not int or val < 1:
+            raise ValueError, _("Number of vcpus must be a postive integer.")
+        if val > maxvcpus:
             raise ValueError, \
-                  _("Number of vcpus must be in the range of 1-%d") % (maxvcpus,)
+                  _("Number of vcpus must be no greater than %d for this vm type.") % maxvcpus
         self._vcpus = val
     vcpus = property(get_vcpus, set_vcpus)
 
@@ -657,84 +509,101 @@ class Guest(object):
         self._cpuset = val
     cpuset = property(get_cpuset, set_cpuset)
 
-    # graphics setup
-    def get_graphics(self):
-        return self._graphics
-    def set_graphics(self, val):
-        def validate_keymap(keymap):
-            if not keymap:
-                return keymap
-            if type(keymap) != type("string"):
-                raise ValueError, _("Keymap must be a string")
-            if len(keymap) > 16:
-                raise ValueError, _("Keymap must be less than 16 characters")
-            if re.match("^[a-zA-Z0-9_-]*$", keymap) == None:
-                raise ValueError, _("Keymap can only contain alphanumeric, '_', or '-' characters")
-            return keymap
+    def get_graphics_dev(self):
+        return self._graphics_dev
+    def set_graphics_dev(self, val):
+        self._graphics_dev = val
+    graphics_dev = property(get_graphics_dev, set_graphics_dev)
 
-        opts = None
-        t = None
+    # DEPRECATED PROPERTIES
+    # Deprecated: Should set graphics_dev.keymap directly
+    def get_keymap(self):
+        if self._graphics_dev is None:
+            return None
+        return self._graphics_dev.keymap
+    def set_keymap(self, val):
+        if self._graphics_dev is not None:
+            self._graphics_dev.keymap = val
+    keymap = property(get_keymap, set_keymap)
+
+    # Deprecated: Should set guest.graphics_dev = VirtualGraphics(...)
+    def get_graphics(self):
+        if self._graphics_dev is None:
+            return { "enabled " : False }
+        return { "enabled" : True, "type" : self._graphics_dev, \
+                 "keymap"  : self._graphics_dev.keymap}
+    def set_graphics(self, val):
+
+        # val can be:
+        #   a dictionary with keys:  enabled, type, port, keymap
+        #   a tuple of the form   : (enabled, type, port, keymap)
+        #                            last 2 optional
+        #                         : "vnc", "sdl", or false
+        port = None
+        gtype = None
+        enabled = False
+        keymap = None
+        gdev = None
         if type(val) == dict:
             if not val.has_key("enabled"):
                 raise ValueError, _("Must specify whether graphics are enabled")
-            self._graphics["enabled"] = val["enabled"]
+            enabled = val["enabled"]
             if val.has_key("type"):
-                t = val["type"]
+                gtype = val["type"]
                 if val.has_key("opts"):
-                    opts = val["opts"]
+                    port = val["opts"]
         elif type(val) == tuple:
-            if len(val) >= 1: self._graphics["enabled"] = val[0]
-            if len(val) >= 2: t = val[1]
-            if len(val) >= 3: opts = val[2]
-            if len(val) >= 4: self._graphics["keymap"] = validate_keymap(val[3])
+            if len(val) >= 1: enabled = val[0]
+            if len(val) >= 2: gtype = val[1]
+            if len(val) >= 3: port = val[2]
+            if len(val) >= 4: keymap = val[3]
         else:
             if val in ("vnc", "sdl"):
-                t = val
-                self._graphics["enabled"] = True
+                gtype = val
+                enabled = True
             else:
-                self._graphics["enabled"] = val
+                enabled = val
 
-        if self._graphics["enabled"] not in (True, False):
+        if enabled not in (True, False):
             raise ValueError, _("Graphics enabled must be True or False")
 
-        if self._graphics["enabled"] == True:
-            if t == "vnc":
-                if self.graphics.has_key("keymap"):
-                    gt = VNCVirtualGraphics(opts, self._graphics["keymap"])
-                else:
-                    gt = VNCVirtualGraphics(opts)
-            elif t == "sdl":
-                gt = SDLVirtualGraphics(opts)
-            else:
-                raise ValueError, _("Unknown graphics type")
-            self._graphics["type"] = gt
+        if enabled == True:
+            gdev = VirtualGraphics(type=gtype)
+            if port:
+                gdev.port = port
+            if keymap:
+                gdev.keymap = keymap
+        self._graphics_dev = gdev
 
     graphics = property(get_graphics, set_graphics)
 
-
-    # Legacy, deprecated properties
-    def get_scratchdir(self):
-        return self._installer.scratchdir
-    scratchdir = property(get_scratchdir)
-
-    def get_boot(self):
-        return self._installer.boot
-    def set_boot(self, val):
-        self._installer.boot = val
-    boot = property(get_boot, set_boot)
-
+    # Deprecated: Should be called from the installer directly
     def get_location(self):
         return self._installer.location
     def set_location(self, val):
         self._installer.location = val
     location = property(get_location, set_location)
 
+    # Deprecated: Should be called from the installer directly
+    def get_scratchdir(self):
+        return self._installer.scratchdir
+    scratchdir = property(get_scratchdir)
+
+    # Deprecated: Should be called from the installer directly
+    def get_boot(self):
+        return self._installer.boot
+    def set_boot(self, val):
+        self._installer.boot = val
+    boot = property(get_boot, set_boot)
+
+    # Deprecated: Should be called from the installer directly
     def get_extraargs(self):
         return self._installer.extraargs
     def set_extraargs(self, val):
         self._installer.extraargs = val
     extraargs = property(get_extraargs, set_extraargs)
 
+    # Deprecated: Should set the installer values directly
     def get_cdrom(self):
         return self._installer.location
     def set_cdrom(self, val):
@@ -749,6 +618,7 @@ class Guest(object):
             self._installer.location = val
         self._installer.cdrom = True
     cdrom = property(get_cdrom, set_cdrom)
+    # END DEPRECATED PROPERTIES
 
 
     def _create_devices(self,progresscb):
@@ -761,31 +631,47 @@ class Guest(object):
     def _get_network_xml(self, install = True):
         """Get the network config in the libvirt XML format"""
         ret = ""
-        for n in self.nics:
+        for n in self._install_nics:
+            if ret:
+                ret += "\n"
             ret += n.get_xml_config()
         return ret
 
     def _get_graphics_xml(self, install = True):
         """Get the graphics config in the libvirt XML format."""
-        ret = ""
-        if self.graphics["enabled"] == False:
-            return ret
-        gt = self.graphics["type"]
-        return gt.get_xml_config()
+        if self._graphics_dev is None:
+            return ""
+        return self._graphics_dev.get_xml_config()
 
     def _get_input_xml(self, install = True):
         """Get the input device config in libvirt XML format."""
         (type,bus) = self.get_input_device()
         return "    <input type='%s' bus='%s'/>" % (type, bus)
 
+    def _get_sound_xml(self):
+        """Get the sound device configuration in libvirt XML format."""
+        xml = ""
+        for sound_dev in self.sound_devs:
+            if xml != "":
+                xml += "\n"
+            xml += sound_dev.get_xml_config()
+        return xml
+
     def _get_device_xml(self, install = True):
-        return """%(disks)s
-%(networks)s
-%(input)s
-%(graphics)s""" % { "disks": self._get_disk_xml(install), \
-        "networks": self._get_network_xml(install), \
-        "input": self._get_input_xml(install), \
-        "graphics": self._get_graphics_xml(install) }
+
+        xml = ""
+        diskxml     = self._get_disk_xml(install)
+        netxml      = self._get_network_xml(install)
+        inputxml    = self._get_input_xml(install)
+        graphicsxml = self._get_graphics_xml(install)
+        soundxml    = self._get_sound_xml()
+        for devxml in [diskxml, netxml, inputxml, graphicsxml, soundxml]:
+            if devxml:
+                if xml:
+                    xml += "\n"
+                xml += devxml
+        return xml
+
 
     def get_config_xml(self, install = True, disk_boot = False):
         if install:
@@ -832,7 +718,8 @@ class Guest(object):
         "action": action }
 
 
-    def start_install(self, consolecb = None, meter = None):
+    def start_install(self, consolecb=None, meter=None, removeOld=False,
+                      wait=True):
         """Do the startup of the guest installation."""
         self.validate_parms()
 
@@ -842,20 +729,34 @@ class Guest(object):
 
         self._prepare_install(meter)
         try:
-            return self._do_install(consolecb, meter)
+            return self._do_install(consolecb, meter, removeOld, wait)
         finally:
             self._installer.cleanup()
 
     def _prepare_install(self, meter):
         self._install_disks = self.disks[:]
         self._install_nics = self.nics[:]
+        self._set_defaults()
 
-    def _do_install(self, consolecb, meter):
+    def _do_install(self, consolecb, meter, removeOld=False, wait=True):
+        vm = None
         try:
-            if self.conn.lookupByName(self.name) is not None:
-                raise RuntimeError, _("Domain named %s already exists!") %(self.name,)
+            vm = self.conn.lookupByName(self.name)
         except libvirt.libvirtError:
-            pass
+           pass
+
+        if vm is not None:
+            if removeOld :
+                try:
+                    if vm.ID() != -1:
+                        logging.info("Destroying image %s" %(self.name))
+                        vm.destroy()
+                    logging.info("Removing old definition for image %s" %(self.name))
+                    vm.undefine()
+                except libvirt.libvirtError, e:
+                    raise RuntimeError, _("Could not remove old vm '%s': %s") %(self.name, str(e))
+            else:
+                raise RuntimeError, _("Domain named %s already exists!") %(self.name,)
 
         child = None
         self._create_devices(meter)
@@ -895,7 +796,7 @@ class Guest(object):
         logging.debug("Saving XML boot config '%s'" % ( boot_xml ))
         self.conn.defineXML(boot_xml)
 
-        if child: # if we connected the console, wait for it to finish
+        if child and wait: # if we connected the console, wait for it to finish
             try:
                 (pid, status) = os.waitpid(child, 0)
             except OSError, (errno, msg):
@@ -912,7 +813,7 @@ class Guest(object):
     def post_install_check(self):
         return self.installer.post_install_check(self)
 
-    def connect_console(self, consolecb):
+    def connect_console(self, consolecb, wait=True):
         logging.debug("Restarted guest, looking to see if it is running")
         # sleep in .25 second increments until either a) we get running
         # domain ID or b) it's been 5 seconds.  this is so that
@@ -940,7 +841,7 @@ class Guest(object):
             logging.debug("Launching console callback")
             child = consolecb(self.domain)
 
-        if child: # if we connected the console, wait for it to finish
+        if child and wait: # if we connected the console, wait for it to finish
             try:
                 (pid, status) = os.waitpid(child, 0)
             except OSError, (errno, msg):
@@ -949,7 +850,6 @@ class Guest(object):
     def validate_parms(self):
         if self.domain is not None:
             raise RuntimeError, _("Domain has already been started!")
-        self._set_defaults()
 
     def _set_defaults(self):
         if self.uuid is None:
