@@ -21,8 +21,10 @@
 import Guest
 import ImageParser
 import CapabilitiesParser as Cap
+from VirtualDisk import VirtualDisk
 import os
 import util
+from virtinst import _virtinst as _
 
 class ImageInstallerException(RuntimeError):
     def __init__(self, msg):
@@ -81,7 +83,7 @@ class ImageInstaller(Guest.Installer):
                     guest.features[f] = False
 
     def _make_disks(self, guest):
-        for m in self.boot_caps.disks:
+        for m in self.boot_caps.drives:
             p = self._abspath(m.disk.file)
             s = None
             if m.disk.size is not None:
@@ -93,19 +95,19 @@ class ImageInstaller(Guest.Installer):
             if m.disk.use == ImageParser.Disk.USE_SYSTEM and not os.path.exists(p):
                 raise ImageInstallerException(_("System disk %s does not exist")
                                               % p)
-            device = Guest.VirtualDisk.DEVICE_DISK
+            device = VirtualDisk.DEVICE_DISK
             if m.disk.format == ImageParser.Disk.FORMAT_ISO:
-                device = Guest.VirtualDisk.DEVICE_CDROM
-            d = Guest.VirtualDisk(p, s,
-                                  device = device,
-                                  type = Guest.VirtualDisk.TYPE_FILE)
+                device = VirtualDisk.DEVICE_CDROM
+            d = VirtualDisk(p, s,
+                            device = device,
+                            type = VirtualDisk.TYPE_FILE)
             if self.boot_caps.type == "xen" and util.is_blktap_capable():
-                d.driver_name = Guest.VirtualDisk.DRIVER_TAP
+                d.driver_name = VirtualDisk.DRIVER_TAP
             d.target = m.target
 
             guest._install_disks.append(d)
 
-    def _get_osblob(self, install, hvm, arch = None, loader = None):
+    def _get_osblob(self, install, hvm, arch = None, loader = None, conn = None):
         osblob = "<os>\n"
 
         if hvm:
@@ -135,7 +137,7 @@ class ImageInstaller(Guest.Installer):
             osblob += "  </os>"
         elif self.boot_caps.loader == "pygrub" or (self.boot_caps.loader is None and self.boot_caps.type == "xen"):
             osblob += "  </os>\n"
-            osblob += "  <bootloader>/usr/bin/pygrub</bootloader>"
+            osblob += "  <bootloader>%s</bootloader>" % util.pygrub_path(conn)
 
         return osblob
 
@@ -143,7 +145,7 @@ class ImageInstaller(Guest.Installer):
         return True
 
     def _abspath(self, p):
-        return os.path.abspath(os.path.join(self.image.base, p))
+        return self.image.abspath(p)
 
 class PlatformMatchException(Exception):
     def __init__(self, msg):
