@@ -21,6 +21,7 @@
 
 import os
 import Guest
+from VirtualDisk import VirtualDisk
 import CapabilitiesParser
 from virtinst import _virtinst as _
 
@@ -29,15 +30,14 @@ class LiveCDInstallerException(Exception):
         Exception.__init__(self, msg)
 
 class LiveCDInstaller(Guest.Installer):
-    def __init__(self, type = "xen", location = None, os_type = None):
-        Guest.Installer.__init__(self, type, location, os_type = None)
+    def __init__(self, type = "xen", location = None, os_type = None,
+                 conn = None):
+        Guest.Installer.__init__(self, type=type, location=location,
+                                 os_type=os_type, conn=conn)
 
     def prepare(self, guest, meter, distro = None):
         self.cleanup()
 
-        if not os.path.exists(self.location):
-            raise LiveCDInstallerException(_("LiveCD image '%s' does not exist") % self.location)
-
         capabilities = CapabilitiesParser.parse(guest.conn.getCapabilities())
 
         found = False
@@ -47,14 +47,25 @@ class LiveCDInstaller(Guest.Installer):
                 break
 
         if not found:
-            raise LiveCDInstallerException(_("HVM virtualisation not supported; cannot boot LiveCD"))
+            raise LiveCDInstallerException(_("Connection does not support HVM virtualisation, cannot boot live CD"))
 
-        disk = Guest.VirtualDisk(self.location,
-                                 device = Guest.VirtualDisk.DEVICE_CDROM,
-                                 readOnly = True)
-        guest._install_disks.insert(0, disk)
+        path = None
+        vol_tuple = None
+        if type(self.location) is tuple:
+            vol_tuple = self.location
+        elif self.location:
+            path = self.location
+        elif not self.cdrom:
+            raise LiveCDInstallerException(_("CDROM media must be specified "
+                                             "for the live CD installer."))
 
-    def _get_osblob(self, install, hvm, arch = None, loader = None):
+        if path or vol_tuple:
+            disk = VirtualDisk(path=path, conn=guest.conn, volName=vol_tuple,
+                               device = VirtualDisk.DEVICE_CDROM,
+                               readOnly = True)
+            guest._install_disks.insert(0, disk)
+
+    def _get_osblob(self, install, hvm, arch = None, loader = None, conn = None):
         if install:
             return None
 
@@ -64,11 +75,11 @@ class LiveCDInstaller(Guest.Installer):
             os_type = "linux"
 
         osblob  = "<os>\n"
-        osblob += "      <type>%s</type>\n" % os_type
+        osblob += "    <type>%s</type>\n" % os_type
         if loader:
-            osblob += "      <loader>%s</loader>\n" % loader
-        osblob += "      <boot dev='cdrom'/>\n"
-        osblob += "    </os>"
+            osblob += "    <loader>%s</loader>\n" % loader
+        osblob += "    <boot dev='cdrom'/>\n"
+        osblob += "  </os>"
 
         return osblob
 
