@@ -24,6 +24,7 @@ import os
 import gzip
 import re
 import tempfile
+import platform
 import ConfigParser
 
 from virtinst import _virtinst as _
@@ -32,10 +33,14 @@ from virtinst import _virtinst as _
 # ISO image, or a kernel+initrd  pair for a particular OS distribution
 class Distro:
 
-    def __init__(self, uri, type=None, scratchdir=None):
+    def __init__(self, uri, type=None, scratchdir=None, arch=None):
         self.uri = uri
         self.type = type
         self.scratchdir = scratchdir
+        if arch == None:
+            arch = platform.machine()
+        self.arch = arch
+        self.treeinfo = None
 
     def acquireBootDisk(self, fetcher, progresscb):
         raise "Not implemented"
@@ -44,42 +49,126 @@ class Distro:
         raise "Not implemented"
 
     def isValidStore(self, fetcher, progresscb):
+        """Determine if uri points to a tree of the store's distro"""
         raise "Not implemented"
 
+    def _hasTreeinfo(self, fetcher, progresscb):
+        # all Red Hat based distros should have .treeinfo, perhaps others
+        # will in time
+        if not (self.treeinfo is None):
+            return True
+
+        if not fetcher.hasFile(".treeinfo"):
+            return False
+
+        logging.debug("Detected .treeinfo file")
+
+        tmptreeinfo = fetcher.acquireFile(".treeinfo", progresscb)
+        try:
+            self.treeinfo = ConfigParser.SafeConfigParser()
+            self.treeinfo.read(tmptreeinfo)
+        finally:
+            os.unlink(tmptreeinfo)
+        return True
+
+    def _getTreeinfoMedia(self, fetcher, progresscb, mediaName):
+        if self.type == "xen":
+            type = "xen"
+        else:
+            type = self.treeinfo.get("general", "arch")
+
+        return self.treeinfo.get("images-%s" % type, mediaName)
+
+
+class GenericDistro(Distro):
+    """Generic distro store. Check well known paths for kernel locations
+       as a last resort if we can't recognize any actual distro"""
+    _xen_paths = [ ("images/xen/vmlinuz",
+                    "images/xen/initrd.img"),           # Fedora
+                 ]
+    _hvm_paths = [ ("images/pxeboot/vmlinuz",
+                    "images/pxeboot/initrd.img"),       # Fedora
+                 ]
+    _iso_paths = [ "images/boot.iso",                   # RH/Fedora
+                   "boot/boot.iso",                     # Suse
+                   "current/images/netboot/mini.iso",   # Debian
+                   "install/images/boot.iso",           # Mandriva
+                 ]
+
+    # Holds values to use when actually pulling down media
+    _valid_kernel_path = None
+    _valid_iso_path = None
+
+    def isValidStore(self, fetcher, progresscb):
+        if self._hasTreeinfo(fetcher, progresscb):
+            # Use treeinfo to pull down media paths
+            if self.type == "xen":
+                type = "xen"
+            else:
+                type = self.treeinfo.get("general", "arch")
+            kernelSection = "images-%s" % type
+            isoSection = "images-%s" % self.treeinfo.get("general", "arch")
+
+            if self.treeinfo.has_section(kernelSection):
+                self._valid_kernel_path = (self._getTreeinfoMedia(fetcher, progresscb, "kernel"), self._getTreeinfoMedia(fetcher, progresscb, "initrd"))
+            if self.treeinfo.has_section(isoSection):
+                self._valid_iso_path = self.treeinfo.get(isoSection, "boot.iso")
+
+        if self.type == "xen":
+            kern_list = self._xen_paths
+        else:
+            kern_list = self._hvm_paths
+
+        # If validated media paths weren't found (no treeinfo), check against
+        # list of media location paths.
+        for kern, init in kern_list:
+            if self._valid_kernel_path == None \
+               and fetcher.hasFile(kern) and fetcher.hasFile(init):
+                self._valid_kernel_path = (kern, init)
+                break
+        for iso in self._iso_paths:
+            if self._valid_iso_path == None \
+               and fetcher.hasFile(iso):
+                self._valid_iso_path = iso
+                break
+
+        if self._valid_kernel_path or self._valid_iso_path:
+            return True
+        return False
+
+    def acquireKernel(self, fetcher, progresscb):
+        if self._valid_kernel_path == None:
+            raise ValueError(_("Could not find a kernel path for virt type "
+                               "'%s'" % self.type))
+
+        kernel = fetcher.acquireFile(self._valid_kernel_path[0], progresscb)
+        try:
+            initrd = fetcher.acquireFile(self._valid_kernel_path[1], progresscb)
+            if fetcher.location.startswith("/"):
+                return (kernel, initrd, "")
+            else:
+                return (kernel, initrd, "method=" + fetcher.location)
+        except:
+            os.unlink(kernel)
+
+    def acquireBootDisk(self, fetcher, progresscb):
+        if self._valid_iso_path == None:
+            raise ValueError(_("Could not find a boot iso path for this tree."))
+
+        return fetcher.acquireFile(self._valid_iso_path, progresscb)
+
 
 # Base image store for any Red Hat related distros which have
 # a common layout
 class RedHatDistro(Distro):
-    def __init__(self, uri, type=None, scratchdir=None):
-        Distro.__init__(self, uri, type, scratchdir)
-        self.treeinfo = None
-
-    def hasTreeinfo(self, fetcher, progresscb):
-        # all Red Hat based distros should have .treeinfo / execute only once
-        if (self.treeinfo is None):
-            if fetcher.hasFile(".treeinfo"):
-                logging.debug("Detected .treeinfo file")
-                tmptreeinfo = fetcher.acquireFile(".treeinfo", progresscb)
-                self.treeinfo = ConfigParser.SafeConfigParser()
-                self.treeinfo.read(tmptreeinfo)
-                return True
-            else:
-                return False
-        else:
-            return True
 
     def acquireKernel(self, fetcher, progresscb):
-        if self.hasTreeinfo(fetcher, progresscb):
-            if self.type == "xen":
-                type = "xen"
-            else:
-                type = self.treeinfo.get("general", "arch")
-
-            kernelpath = self.treeinfo.get("images-%s" % type, "kernel")
-            initrdpath = self.treeinfo.get("images-%s" % type, "initrd")
+        if self._hasTreeinfo(fetcher, progresscb):
+            kernelpath = self._getTreeinfoMedia(fetcher, progresscb, "kernel")
+            initrdpath = self._getTreeinfoMedia(fetcher, progresscb, "initrd")
         else:
             # fall back to old code
-            if self.type is None:
+            if self.type is None or self.type == "hvm":
                 kernelpath = "images/pxeboot/vmlinuz"
                 initrdpath = "images/pxeboot/initrd.img"
             else:
@@ -98,19 +187,17 @@ class RedHatDistro(Distro):
             os.unlink(kernel)
 
     def acquireBootDisk(self, fetcher, progresscb):
-        if self.hasTreeinfo(fetcher, progresscb):
-            if self.type == "xen":
-                type = "xen"
-            else:
-                type = self.treeinfo.get("general", "arch")
-            return fetcher.acquireFile(self.treeinfo.get("images-%s" % type, "boot.iso"), progresscb)
+        if self._hasTreeinfo(fetcher, progresscb):
+            return fetcher.acquireFile(self._getTreeinfoMedia(fetcher,
+                                                              progresscb,
+                                                              "boot.iso"))
         else:
             return fetcher.acquireFile("images/boot.iso", progresscb)
 
 # Fedora distro check
 class FedoraDistro(RedHatDistro):
     def isValidStore(self, fetcher, progresscb):
-        if self.hasTreeinfo(fetcher, progresscb):
+        if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*Fedora.*", self.treeinfo.get("general", "family"))
             return (m != None)
         else:
@@ -122,7 +209,7 @@ class FedoraDistro(RedHatDistro):
 # Red Hat Enterprise Linux distro check
 class RHELDistro(RedHatDistro):
     def isValidStore(self, fetcher, progresscb):
-        if self.hasTreeinfo(fetcher, progresscb):
+        if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*Red Hat Enterprise Linux.*", self.treeinfo.get("general", "family"))
             return (m != None)
         else:
@@ -141,7 +228,7 @@ class RHELDistro(RedHatDistro):
 # CentOS distro check
 class CentOSDistro(RedHatDistro):
     def isValidStore(self, fetcher, progresscb):
-        if self.hasTreeinfo(fetcher, progresscb):
+        if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*CentOS.*", self.treeinfo.get("general", "family"))
             return (m != None)
         else:
@@ -151,15 +238,57 @@ class CentOSDistro(RedHatDistro):
                 return True
             return False
 
+# Scientific Linux distro check
+class SLDistro(RedHatDistro):
+    def isValidStore(self, fetcher, progresscb):
+        if fetcher.hasFile("SL"):
+            logging.debug("Detected a Scientific Linux distro")
+            return True
+        return False
+
 
 
 # Suse  image store is harder - we fetch the kernel RPM and a helper
 # RPM and then munge bits together to generate a initrd
 class SuseDistro(Distro):
+    def __init__(self, uri, type=None, scratchdir=None, arch=None):
+        Distro.__init__(self, uri, type, scratchdir, arch)
+        if len(self.arch) == 4 and self.arch[0] == 'i' \
+           and self.arch[2:] == "86":
+            self.arch = "i386"
+
     def acquireBootDisk(self, fetcher, progresscb):
         return fetcher.acquireFile("boot/boot.iso", progresscb)
 
     def acquireKernel(self, fetcher, progresscb):
+        kernel_args = ""
+        if not fetcher.location.startswith("/"):
+            kernel_args = "method=" + fetcher.location
+
+        # If installing a fullvirt guest
+        if self.type is None or self.type == "hvm":
+            # Tested with Opensuse 10, 11, and sles 10
+            kernel = fetcher.acquireFile("boot/%s/loader/linux" % self.arch,
+                                         progresscb)
+            initrd = fetcher.acquireFile("boot/%s/loader/initrd" % self.arch,
+                                         progresscb)
+            return (kernel, initrd, kernel_args)
+
+        # Else we are looking for a paravirt kernel
+        if fetcher.hasFile("boot/%s/vmlinuz-xen" % self.arch):
+            # Should match opensuse > 10.2 and sles 10
+            kernel = fetcher.acquireFile("boot/%s/vmlinuz-xen" % self.arch,
+                                         progresscb)
+            initrd = fetcher.acquireFile("boot/%s/initrd-xen" % self.arch,
+                                         progresscb)
+            return (kernel, initrd, kernel_args)
+
+        # For Opensuse <= 10.2, we need to perform some heinous stuff
+        logging.debug("Trying Opensuse 10 PV rpm hacking")
+        return self._findXenRPMS(fetcher, progresscb)
+
+
+    def _findXenRPMS(self, fetcher, progresscb):
         kernelrpm = None
         installinitrdrpm = None
         filelist = None
@@ -167,14 +296,14 @@ class SuseDistro(Distro):
             # There is no predictable filename for kernel/install-initrd RPMs
             # so we have to grok the filelist and find them
             filelist = fetcher.acquireFile("ls-lR.gz", progresscb)
-            (kernelrpmname, installinitrdrpmname) = self.extractRPMNames(filelist)
+            (kernelrpmname, initrdrpmname) = self._extractRPMNames(filelist)
 
             # Now fetch the two RPMs we want
             kernelrpm = fetcher.acquireFile(kernelrpmname, progresscb)
-            installinitrdrpm = fetcher.acquireFile(installinitrdrpmname, progresscb)
+            installinitrdrpm = fetcher.acquireFile(initrdrpmname, progresscb)
 
             # Process the RPMs to extract the kernel & generate an initrd
-            return self.buildKernelInitrd(fetcher, kernelrpm, installinitrdrpm, progresscb)
+            return self._buildKernelInitrd(fetcher, kernelrpm, installinitrdrpm, progresscb)
         finally:
             if filelist is not None:
                 os.unlink(filelist)
@@ -186,20 +315,21 @@ class SuseDistro(Distro):
     # We need to parse the ls-lR.gz file, looking for the kernel &
     # install-initrd RPM entries - capturing the directory they are
     # in and the version'd filename.
-    def extractRPMNames(self, filelist):
+    def _extractRPMNames(self, filelist):
         filelistData = gzip.GzipFile(filelist, mode = "r")
         try:
-            arch = os.uname()[4]
-            arches = [arch]
+            arches = [self.arch]
             # On i686 arch, we also look under i585 and i386 dirs
             # in case the RPM is built for a lesser arch. We also
             # need the PAE variant (for Fedora dom0 at least)
             #
             # XXX shouldn't hard code that dom0 is PAE
-            if arch == "i686":
+            if self.arch == "i386":
                 arches.append("i586")
-                arches.append("i386")
+                arches.append("i686")
                 kernelname = "kernel-xenpae"
+            else:
+                kernelname = "kernel-xen"
 
             installinitrdrpm = None
             kernelrpm = None
@@ -239,7 +369,7 @@ class SuseDistro(Distro):
     # booting the installer.
     #
     # Yes, this is crazy ass stuff :-)
-    def buildKernelInitrd(self, fetcher, kernelrpm, installinitrdrpm, progresscb):
+    def _buildKernelInitrd(self, fetcher, kernelrpm, installinitrdrpm, progresscb):
         progresscb.start(text=_("Building initrd"), size=11)
         progresscb.update(1)
         cpiodir = tempfile.mkdtemp(prefix="virtinstcpio.", dir=self.scratchdir)
@@ -359,7 +489,7 @@ class SuseDistro(Distro):
 class DebianDistro(Distro):
     def isValidStore(self, fetcher, progresscb):
         # Don't support any paravirt installs
-        if self.type is not None:
+        if self.type is not None and self.type != "hvm":
             return False
 
         file = None
@@ -400,21 +530,21 @@ class DebianDistro(Distro):
 class UbuntuDistro(Distro):
     def isValidStore(self, fetcher, progresscb):
         # Don't support any paravirt installs
-        if self.type is not None:
+        if self.type is not None and self.type != "hvm":
             return False
         return False
 
 class GentooDistro(Distro):
     def isValidStore(self, fetcher, progresscb):
         # Don't support any paravirt installs
-        if self.type is not None:
+        if self.type is not None and self.type != "hvm":
             return False
         return False
 
 class MandrivaDistro(Distro):
     def isValidStore(self, fetcher, progresscb):
         # Don't support any paravirt installs
-        if self.type is not None:
+        if self.type is not None and self.type != "hvm":
             return False
 
         # Mandriva websites / media appear to have a VERSION
