@@ -30,33 +30,13 @@ class ParaVirtGuest(Guest.XenGuest):
         if not installer:
             installer = DistroManager.DistroInstaller(type = type, os_type = "linux")
         Guest.Guest.__init__(self, type, connection, hypervisorURI, installer)
-        if self.type == "xen":
-            self.disknode = "xvd"
-        else:
-            # XXX hack for libvirt + xenner limitation - it should
-            # allow xvdNNN, but rejects it. Remove this when
-            # libvirt is fixed
-            self.disknode = "hd"
+        self.disknode = "xvd"
 
     def _get_osblob(self, install):
-        return self.installer._get_osblob(install, hvm = False)
-
-    def _connectSerialConsole(self):
-        cmd = ["/usr/bin/virsh", "console", "%s" %(self.domain.ID(),)]
-        child = os.fork()
-        if (not child):
-            os.execvp(cmd[0], cmd)
-            os._exit(1)
-        return child
+        return self.installer._get_osblob(install, hvm = False, conn = self.conn)
 
     def get_input_device(self):
-        if self.type == "xen":
-            return ("mouse", "xen")
-        else:
-            # XXX hack for libvirt + xenner limitation - it should
-            # allow mouse+xen, but rejects it. Remove this when
-            # libvirt is fixed
-            return ("mouse", "ps2")
+        return ("mouse", "xen")
 
     def validate_parms(self):
         if not self.location and not self.boot:
@@ -73,21 +53,17 @@ class ParaVirtGuest(Guest.XenGuest):
     def _get_disk_xml(self, install = True):
         """Get the disk config in the libvirt XML format"""
         ret = ""
-        nodes = {}
-        for i in range(16):
-            n = "%s%c" % (self.disknode, ord('a') + i)
-            nodes[n] = None
+        used_targets = []
+        for disk in self._install_disks:
+            if not disk.bus:
+                disk.bus = "xen"
+            used_targets.append(disk.generate_target(used_targets))
+
         for d in self._install_disks:
             if d.transient and not install:
                 continue
-            target = d.target
-            if target is None:
-                for t in sorted(nodes.keys()):
-                    if nodes[t] is None:
-                        target = t
-                        break
-            if target is None or nodes[target] is not None:
-                raise ValueError, _("Can't use more than 16 disks on a PV guest")
-            nodes[target] = True
-            ret += d.get_xml_config(target)
+
+            if ret:
+                ret += "\n"
+            ret += d.get_xml_config(d.target)
         return ret
