@@ -25,7 +25,9 @@ def _virtinst(msg):
     return gettext.dgettext(gettext_app, msg)
 
 import util
-from Guest import Guest, VirtualDisk, VirtualNetworkInterface, XenGuest, XenDisk, XenNetworkInterface
+from Guest import Guest, VirtualNetworkInterface, XenGuest, \
+                  XenNetworkInterface, VirtualGraphics, VirtualAudio
+from VirtualDisk import VirtualDisk, XenDisk
 from FullVirtGuest import FullVirtGuest
 from ParaVirtGuest import ParaVirtGuest
 from DistroManager import DistroInstaller, PXEInstaller
